<?php

declare(strict_types=1);

namespace Zyimm\Sms\Contracts;

use Zyimm\Sms\Exceptions\DriverErrorException;

/**
 * @property string[]                    $senders
 * @property string                      $strategy
 * @property null|string                 $from
 * @property MobileNumberInterface $to
 * @property null|string                 $content
 * @property null|string                 $template
 * @property null|string                 $signature
 * @property array                       $data
 */
interface SmsableInterface
{
    /**
     * Set the SMS message sender number.
     *
     * @return $this
     */
    public function from(string $from): SmsableInterface;

    /**
     * Set the SMS message recipient number.
     *
     * @return $this
     */
    public function to(MobileNumberInterface $to): SmsableInterface;

    /**
     * Set the SMS message content.
     *
     * @return $this
     */
    public function content(string $content): SmsableInterface;

    /**
     * Set the SMS message template.
     *
     * @return $this
     */
    public function template(string $template): SmsableInterface;

    /**
     * Set the SMS message signature.
     *
     * @return $this
     */
    public function signature(string $signature): SmsableInterface;

    /**
     * Set the SMS message data.
     *
     * @param array|string $key
     * @param null|mixed $value
     *
     * @return $this
     */
    public function with($key, $value = null): SmsableInterface;

    /**
     * Set the strategy.
     *
     * @return $this
     */
    public function strategy(string $class): SmsableInterface;

    /**
     * Set the list of sender name of the SMS message.
     *
     * @param string[] $names
     *
     * @return $this
     */
    public function senders(array $names): SmsableInterface;

    /**
     * Set the sender name of the SMS message. This will override `$senders` property value.
     *
     * @return $this
     */
    public function sender(string $name): SmsableInterface;

    /**
     * Send the SMS message immediately.
     *
     * @throws DriverErrorException
     */
    public function send(?SenderInterface $sender = null): array;

    /**
     * Queue the SMS message for sending.
     */
    public function queue(?string $queue = null): bool;

    /**
     * Deliver the queued SMS message after the given delay.
     */
    public function later(int $delay, ?string $queue = null): bool;
}
