<?php

declare(strict_types=1);

namespace Zyimm\Sms;

use Hyperf\AsyncQueue\Driver\DriverFactory;
use Hyperf\Contract\CompressInterface;
use Hyperf\Contract\UnCompressInterface;
use Hyperf\Utils\ApplicationContext;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use Zyimm\Sms\Contracts\MobileNumberInterface;
use Zyimm\Sms\Contracts\SenderInterface;
use Zyimm\Sms\Contracts\SmsableInterface;
use Zyimm\Sms\Contracts\SmsManagerInterface;
use Zyimm\Sms\Strategies\OrderStrategy;

abstract class Smsable implements SmsableInterface, CompressInterface, UnCompressInterface
{
    /**
     * @var string
     */
    public string $strategy = OrderStrategy::class;

    /**
     * @var string[]
     */
    public array $senders = [];

    /**
     * @var string
     */
    public string $sender;

    /**
     * @var string
     */
    public string $from;

    /**
     * @var MobileNumberInterface
     */
    public MobileNumberInterface $to;

    /**
     * @var string
     */
    public string $content;

    /**
     * @var string
     */
    public string $template;

    /**
     * @var string
     */
    public string $signature;

    /**
     * @var array
     */
    public array $data = [];

    public function from(string $from): Smsable
    {
        $this->from = $from;

        return $this;
    }

    public function to(MobileNumberInterface $to): Smsable
    {
        $this->to = $to;

        return $this;
    }

    public function content(string $content): Smsable
    {
        $this->content = $content;

        return $this;
    }

    public function template(string $template): Smsable
    {
        $this->template = $template;

        return $this;
    }

    public function signature(string $signature): Smsable
    {
        $this->signature = $signature;

        return $this;
    }

    public function with($key, $value = null): Smsable
    {
        if (is_array($key)) {
            $this->data = array_merge($this->data, $key);
        } elseif (is_string($key)) {
            $this->data[$key] = $value;
        }

        return $this;
    }

    public function strategy(string $class): Smsable
    {
        $this->strategy = $class;

        return $this;
    }

    public function senders(array $names): Smsable
    {
        $this->senders = $names;

        return $this;
    }

    public function sender(string $name): Smsable
    {
        $this->sender = $name;

        return $this;
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function send(?SenderInterface $sender = null): array
    {
        return $sender instanceof SenderInterface
            ? $sender->send($this)
            : ApplicationContext::getContainer()->get(SmsManagerInterface::class)->sendNow($this);
    }

    /**
     * queue
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function queue(?string $queue = null): bool
    {
        return $this->pushQueuedJob($this->newQueuedJob(), $queue);
    }

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function later(int $delay, ?string $queue = null): bool
    {
        return $this->pushQueuedJob($this->newQueuedJob(), $queue, $delay);
    }

    /**
     * @return static
     */
    public function uncompress(): CompressInterface
    {
        foreach ($this as $key => $value) {
            if ($value instanceof UnCompressInterface) {
                $this->{$key} = $value->uncompress();
            }
        }

        return $this;
    }

    /**
     * @return static
     */
    public function compress(): UnCompressInterface
    {
        foreach ($this as $key => $value) {
            if ($value instanceof CompressInterface) {
                $this->{$key} = $value->compress();
            }
        }

        return $this;
    }

    /**
     * Push the queued SMS message job onto the queue.
     *
     * @param  QueuedSmsableJob  $job
     * @param  string|null  $queue
     * @param  int|null  $delay
     * @return bool
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    protected function pushQueuedJob(QueuedSmsableJob $job, ?string $queue = null, ?int $delay = null): bool
    {
        $queue = $queue ?: (property_exists($this, 'queue') ? $this->queue : array_key_first(config('async_queue')));

        return ApplicationContext::getContainer()->get(DriverFactory::class)->get($queue)->push($job, (int) $delay);
    }

    /**
     * Make the queued SMS message job instance.
     */
    protected function newQueuedJob(): QueuedSmsableJob
    {
        return new QueuedSmsableJob($this);
    }
}
